/**
* @license Apache-2.0
*
* Copyright (c) 2018 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MAIN //

/**
* Polyfill for changing the current working directory in environments having no native support for doing so.
*
* @param {string} path - desired working directory
* @returns {Error} error object
*
* @example
* var err = chdir( __dirname );
* if ( err ) {
*     console.error( err.message );
* }
*/
function chdir() {
	return new Error( 'invalid operation. A browser environment has no support for changing the current working directory.' );
}


// EXPORTS //

module.exports = chdir;
