/*
* @license Apache-2.0
*
* Copyright (c) 2019 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

// TypeScript Version: 2.0

/**
* Node version.
*
* @example
* var semver = require( `semver` );
*
* if ( semver.lt( NODE_VERSION, '1.0.0' ) ) {
*    console.log( 'Running on a pre-io.js version...' );
* }
* else if ( semver.lt( NODE_VERSION, '4.0.0' ) ) {
*    console.log( 'Running on an io.js version...' );
* }
* else {
*    console.log( 'Running on a post-io.js version...' );
* }
*/
declare const NODE_VERSION: string;


// EXPORTS //

export = NODE_VERSION;
